package ink.wgink.database.controller.app.api.mysql;

import ink.wgink.annotation.CheckRequestBodyAnnotation;
import ink.wgink.common.base.DefaultBaseController;
import ink.wgink.database.pojo.dtos.mysql.MysqlDTO;
import ink.wgink.database.pojo.vos.mysql.MysqlVO;
import ink.wgink.database.service.mysql.IMysqlService;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.result.ErrorResult;
import ink.wgink.pojo.result.SuccessResult;
import ink.wgink.pojo.result.SuccessResultData;
import ink.wgink.pojo.result.SuccessResultList;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: MysqlAppController
 * @Description: mysql连接
 * @Author: CodeFactory
 * @Date: 2021-10-26 23:05:32
 * @Version: 3.0
 **/
@Api(tags = ISystemConstant.API_TAGS_APP_PREFIX + "mysql连接接口")
@RestController
@RequestMapping(ISystemConstant.APP_PREFIX + "/mysql")
public class MysqlAppController extends DefaultBaseController {

    @Autowired
    private IMysqlService mysqlService;

    @ApiOperation(value = "新增mysql连接", notes = "新增mysql连接接口")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "token", value = "token", paramType = "header")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @PostMapping("save")
    @CheckRequestBodyAnnotation
    public SuccessResult save(@RequestHeader("token") String token, @RequestBody MysqlVO mysqlVO) {
        mysqlService.save(token, mysqlVO);
        return new SuccessResult();
    }

    @ApiOperation(value = "删除mysql连接(id列表)", notes = "删除mysql连接(id列表)接口")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "token", value = "token", paramType = "header"),
        @ApiImplicitParam(name = "ids", value = "ID列表，用下划线分隔", paramType = "path", example = "1_2_3")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @DeleteMapping("remove/{ids}")
    public SuccessResult remove(@RequestHeader("token") String token, @PathVariable("ids") String ids) {
        mysqlService.remove(token, Arrays.asList(ids.split("\\_")));
        return new SuccessResult();
    }

    @ApiOperation(value = "修改mysql连接", notes = "修改mysql连接接口")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "token", value = "token", paramType = "header"),
        @ApiImplicitParam(name = "mysqlId", value = "mysql连接ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @PutMapping("updatemysql/{mysqlId}")
    @CheckRequestBodyAnnotation
    public SuccessResult updateMysql(@RequestHeader("token") String token, @PathVariable("mysqlId") String mysqlId, @RequestBody MysqlVO mysqlVO) {
        mysqlService.update(token, mysqlId, mysqlVO);
        return new SuccessResult();
    }

    @ApiOperation(value = "mysql连接详情(通过ID)", notes = "mysql连接详情(通过ID)接口")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "token", value = "token", paramType = "header"),
        @ApiImplicitParam(name = "mysqlId", value = "mysql连接ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("get/{mysqlId}")
    public MysqlDTO get(@RequestHeader("token") String token, @PathVariable("mysqlId") String mysqlId) {
        return mysqlService.get(mysqlId);
    }

    @ApiOperation(value = "mysql连接列表", notes = "mysql连接列表接口")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "token", value = "token", paramType = "header")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("list")
    public List<MysqlDTO> list(@RequestHeader("token") String token) {
        Map<String, Object> params = requestParams();
        return mysqlService.list(params);
    }

    @ApiOperation(value = "mysql连接分页列表", notes = "mysql连接分页列表接口")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "token", value = "token", paramType = "header"),
        @ApiImplicitParam(name = "page", value = "当前页码", paramType = "query", dataType = "int", defaultValue = "1"),
        @ApiImplicitParam(name = "rows", value = "显示数量", paramType = "query", dataType = "int", defaultValue = "20"),
        @ApiImplicitParam(name = "keywords", value = "关键字", paramType = "query", dataType = "String"),
        @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query", dataType = "String"),
        @ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query", dataType = "String")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listpagemysql")
    public SuccessResultList<List<MysqlDTO>> listPage(@RequestHeader("token") String token, ListPage page) {
        Map<String, Object> params = requestParams();
        page.setParams(params);
        return mysqlService.listPage(page);
    }

    @ApiOperation(value = "mysql连接统计", notes = "mysql连接统计接口")
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("count")
    SuccessResultData<Integer> count() {
        Map<String, Object> params = requestParams();
        return new SuccessResultData<>(mysqlService.count(params));
    }

}