package ink.wgink.database.service.mysql;

import ink.wgink.database.pojo.bos.mysql.MysqlBO;
import ink.wgink.database.pojo.dtos.mysql.MysqlDTO;
import ink.wgink.database.pojo.pos.mysql.MysqlPO;
import ink.wgink.database.pojo.vos.mysql.MysqlVO;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.result.SuccessResultList;

import java.util.List;
import java.util.Map;

/**
 * @ClassName: IMysqlService
 * @Description: mysql连接
 * @Author: CodeFactory
 * @Date: 2021-10-26 23:05:32
 * @Version: 3.0
 **/
public interface IMysqlService {

    /**
     * 新增mysql连接
     *
     * @param mysqlVO
     * @return
     */
    void save(MysqlVO mysqlVO);

    /**
     * 新增mysql连接
     *
     * @param token
     * @param mysqlVO
     * @return
     */
    void save(String token, MysqlVO mysqlVO);

    /**
     * 新增mysql连接
     *
     * @param mysqlVO
     * @return mysqlId
     */
    String saveReturnId(MysqlVO mysqlVO);

    /**
     * 新增mysql连接
     *
     * @param token
     * @param mysqlVO
     * @return mysqlId
     */
    String saveReturnId(String token, MysqlVO mysqlVO);

    /**
     * 删除mysql连接
     *
     * @param ids id列表
     * @return
     */
    void remove(List<String> ids);


    /**
     * 删除mysql连接
     *
     * @param token
     * @param ids id列表
     * @return
     */
    void remove(String token, List<String> ids);

    /**
     * 删除mysql连接（物理删除）
     *
     * @param ids id列表
     */
    void delete(List<String> ids);

    /**
     * 修改mysql连接
     *
     * @param mysqlId
     * @param mysqlVO
     * @return
     */
    void update(String mysqlId, MysqlVO mysqlVO);

    /**
     * 修改mysql连接
     *
     * @param token
     * @param mysqlId
     * @param mysqlVO
     * @return
     */
    void update(String token, String mysqlId, MysqlVO mysqlVO);

    /**
     * mysql连接详情
     *
     * @param params 参数Map
     * @return
     */
    MysqlDTO get(Map<String, Object> params);

    /**
     * mysql连接详情
     *
     * @param mysqlId
     * @return
     */
    MysqlDTO get(String mysqlId);

    /**
     * mysql连接详情
     *
     * @param params 参数Map
     * @return
     */
    MysqlBO getBO(Map<String, Object> params);

    /**
     * mysql连接详情
     *
     * @param mysqlId
     * @return
     */
    MysqlBO getBO(String mysqlId);

    /**
     * mysql连接详情
     *
     * @param params 参数Map
     * @return
     */
    MysqlPO getPO(Map<String, Object> params);

    /**
     * mysql连接详情
     *
     * @param mysqlId
     * @return
     */
    MysqlPO getPO(String mysqlId);

    /**
     * mysql连接列表
     *
     * @param params
     * @return
     */
    List<MysqlDTO> list(Map<String, Object> params);

    /**
     * mysql连接列表
     *
     * @param params
     * @return
     */
    List<MysqlBO> listBO(Map<String, Object> params);

    /**
     * mysql连接列表
     *
     * @param params
     * @return
     */
    List<MysqlPO> listPO(Map<String, Object> params);

    /**
     * mysql连接分页列表
     *
     * @param page
     * @return
     */
    SuccessResultList<List<MysqlDTO>> listPage(ListPage page);

    /**
     * mysql连接统计
     *
     * @param params
     * @return
     */
    Integer count(Map<String, Object> params);

}